@@ -187,15 +187,15 @@ class MainWindow:
                                    _("Automatically detect and recognize all pages"),
                                    tool_items['recognizeDocument']),
                                   ('OCRFeederReconPage', None,
-                                   _('_Recognize Page'), '<control><shift>g',
+                                   _('R_ecognize Page'), '<control><shift>g',
                                    _("Automatically detect and recognize the current page"),
                                    tool_items['recognizePage']),
                                   ('RecognizeAreas', None,
-                                   _('Recognize Selected _Areas'), '<control><shift>r',
+                                   _('Recognize _Selected Areas'), '<control><shift>r',
                                    _("Recognize Selected Areas"),
                                    menu_items['recognize_areas']),
                                   ('SelectAllAreas', None,
-                                   _('Select All _Areas'), '<control><shift>a',
+                                   _('Select _All Areas'), '<control><shift>a',
                                    _("Select all content areas"),
                                    menu_items['select_all_areas']),
                                   ('SelectPreviousArea', gtk.STOCK_GO_BACK,
